@@ -350,7 +350,7 @@ e_cell_combo_show_popup			(ECellCombo	*ecc, int row, int view_col)
 	gdk_window_resize (ecc->popup_window->window, width, height);
 	gtk_widget_show (ecc->popup_window);
 
-	E_CELL_POPUP (ecc)->popup_shown = TRUE;
+	e_cell_popup_set_shown (E_CELL_POPUP (ecc), TRUE);
 	d(g_print("%s: popup_shown = TRUE\n", __FUNCTION__));
 }
 
@@ -520,7 +520,7 @@ e_cell_combo_button_press		(GtkWidget	*popup_window,
 	gdk_pointer_ungrab (event->button.time);
 	gtk_widget_hide (ecc->popup_window);
 
-	E_CELL_POPUP (ecc)->popup_shown = FALSE;
+	e_cell_popup_set_shown (E_CELL_POPUP (ecc), FALSE);
 	d(g_print("%s: popup_shown = FALSE\n", __FUNCTION__));
 
 	/* We don't want to update the cell here. Since the list is in browse
@@ -561,7 +561,7 @@ e_cell_combo_button_release		(GtkWidget	*popup_window,
 	gdk_pointer_ungrab (event->time);
 	gtk_widget_hide (ecc->popup_window);
 
-	E_CELL_POPUP (ecc)->popup_shown = FALSE;
+	e_cell_popup_set_shown (E_CELL_POPUP (ecc), FALSE);
 	d(g_print("%s: popup_shown = FALSE\n", __FUNCTION__));
 
 	e_cell_combo_update_cell (ecc);
@@ -590,7 +590,7 @@ e_cell_combo_key_press			(GtkWidget	*popup_window,
 	gdk_pointer_ungrab (event->time);
 	gtk_widget_hide (ecc->popup_window);
 
-	E_CELL_POPUP (ecc)->popup_shown = FALSE;
+	e_cell_popup_set_shown (E_CELL_POPUP (ecc), FALSE);
 	d(g_print("%s: popup_shown = FALSE\n", __FUNCTION__));
 
 	if (event->keyval != GDK_Escape)
