@@ -1,13 +1,26 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * e-tree.c: A graphical view of a tree.
+/* 
+ * e-tree.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Miguel de Icaza (miguel@ximian.com)
+ * Authors:
  *   Chris Lahey <clahey@ximian.com>
  *
- * Copyright 1999, 2000, 2001, Ximian, Inc
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -45,6 +58,14 @@
 
 static GtkObjectClass *parent_class;
 
+#define d(x)
+
+#if d(!)0
+#define e_table_item_leave_edit_(x) (e_table_item_leave_edit((x)), g_print ("%s: e_table_item_leave_edit\n", __FUNCTION__))
+#else
+#define e_table_item_leave_edit_(x) (e_table_item_leave_edit((x)))
+#endif
+
 enum {
 	CURSOR_CHANGE,
 	CURSOR_ACTIVATED,
@@ -516,10 +537,12 @@ static void
 item_cursor_activated (ETableItem *eti, int row, ETree *et)
 {
 	ETreePath path = e_tree_table_adapter_node_at_row(et->priv->etta, row);
-	path = e_tree_sorted_view_to_model_path(et->priv->sorted, path);
+	if (path)
+		path = e_tree_sorted_view_to_model_path(et->priv->sorted, path);
 	gtk_signal_emit (GTK_OBJECT (et),
 			 et_signals [CURSOR_ACTIVATED],
 			 row, path);
+	d(g_print("%s: Emitted CURSOR_ACTIVATED signal on row: %d and path: 0x%p\n", __FUNCTION__, row, path));
 }
 
 static void
@@ -685,19 +708,55 @@ et_canvas_realize (GtkWidget *canvas, ETree *e_tree)
 }
 
 static gint
-et_canvas_button_press (GtkWidget *canvas, GdkEventButton *event, ETree *e_tree)
+et_canvas_root_event (GnomeCanvasItem *root, GdkEvent *event, ETree *e_tree)
 {
-	if (GTK_WIDGET_HAS_FOCUS(canvas)) {
-		GnomeCanvasItem *item = GNOME_CANVAS(canvas)->focused_item;
+	switch (event->type) {
+	case GDK_BUTTON_PRESS:
+	case GDK_2BUTTON_PRESS:
+	case GDK_BUTTON_RELEASE:
+		if (event->button.button != 4 && event->button.button != 5) {
+			if (GTK_WIDGET_HAS_FOCUS(root->canvas)) {
+				GnomeCanvasItem *item = GNOME_CANVAS(root->canvas)->focused_item;
 
-		if (E_IS_TABLE_ITEM(item)) {
-			e_table_item_leave_edit(E_TABLE_ITEM(item));
+				if (E_IS_TABLE_ITEM(item)) {
+					e_table_item_leave_edit_(E_TABLE_ITEM(item));
+					return TRUE;
+				}
+			}
 		}
+		break;
+	default:
+		break;
 	}
 
 	return FALSE;
 }
 
+/* Handler for focus events in the table_canvas; we have to repaint ourselves
+ * and give the focus to some ETableItem.
+ */
+static gint
+table_canvas_focus_event_cb (GtkWidget *widget, GdkEventFocus *event, gpointer data)
+{
+	GnomeCanvas *canvas;
+	ETree *tree;
+
+	gtk_widget_queue_draw (widget);
+
+	if (!event->in)
+		return TRUE;
+
+	canvas = GNOME_CANVAS (widget);
+	tree = E_TREE (data);
+
+	if (!canvas->focused_item) {
+		e_table_item_set_cursor (E_TABLE_ITEM (tree->priv->item), 0, 0);
+		gnome_canvas_item_grab_focus (tree->priv->item);
+	}
+
+	return TRUE;
+}
+
 static void
 e_tree_setup_table (ETree *e_tree)
 {
@@ -707,10 +766,10 @@ e_tree_setup_table (ETree *e_tree)
 		GTK_SIGNAL_FUNC (tree_canvas_size_allocate), e_tree);
 	gtk_signal_connect (
 		GTK_OBJECT (e_tree->priv->table_canvas), "focus_in_event",
-		GTK_SIGNAL_FUNC (gtk_widget_queue_draw), e_tree);
+		GTK_SIGNAL_FUNC (table_canvas_focus_event_cb), e_tree);
 	gtk_signal_connect (
 		GTK_OBJECT (e_tree->priv->table_canvas), "focus_out_event",
-		GTK_SIGNAL_FUNC (gtk_widget_queue_draw), e_tree);
+		GTK_SIGNAL_FUNC (table_canvas_focus_event_cb), e_tree);
 
 	gtk_signal_connect (
 		GTK_OBJECT (e_tree->priv->table_canvas), "drag_begin",
@@ -756,8 +815,8 @@ e_tree_setup_table (ETree *e_tree)
 		GTK_OBJECT(e_tree->priv->table_canvas), "realize",
 		GTK_SIGNAL_FUNC(et_canvas_realize), e_tree);
 	gtk_signal_connect (
-		GTK_OBJECT(e_tree->priv->table_canvas), "button_press_event",
-		GTK_SIGNAL_FUNC(et_canvas_button_press), e_tree);
+		GTK_OBJECT(gnome_canvas_root (e_tree->priv->table_canvas)), "event",
+		GTK_SIGNAL_FUNC(et_canvas_root_event), e_tree);
 
 	et_build_item(e_tree);
 }
