@@ -861,6 +861,35 @@ g_utf8_get_char_extended (const gchar *p, int max_len)
   return wc;
 }
 
+/**
+ * g_utf8_get_char_validated:
+ * @p: a pointer to Unicode character encoded as UTF-8
+ * @max_len: the maximum number of bytes to read, or -1, for no maximum.
+ * 
+ * Convert a sequence of bytes encoded as UTF-8 to a Unicode character.
+ * This function checks for incomplete characters, for invalid characters
+ * such as characters that are out of the range of Unicode, and for
+ * overlong encodings of valid characters.
+ * 
+ * Return value: the resulting character. If @p points to a partial
+ *    sequence at the end of a string that could begin a valid character,
+ *    returns (gunichar)-2; otherwise, if @p does not point to a valid
+ *    UTF-8 encoded Unicode character, returns (gunichar)-1.
+ **/
+gunichar
+g_utf8_get_char_validated (const  gchar *p,
+			   gssize max_len)
+{
+  gunichar result = g_utf8_get_char_extended (p, max_len);
+
+  if (result & 0x80000000)
+    return result;
+  else if (!UNICODE_VALID (result))
+    return (gunichar)-1;
+  else
+    return result;
+}
+
 /**
  * g_utf8_validate:
  * @str: a pointer to character data
@@ -884,7 +913,7 @@ g_utf8_get_char_extended (const gchar *p, int max_len)
  **/
 gboolean
 g_utf8_validate (const gchar  *str,
-                 gint          max_len,
+                 gssize       max_len,
                  const gchar **end)
 {
 
