@@ -1,3 +1,26 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* 
+ * e-util.h
+ * Copyright 2000, 2001, Ximian, Inc.
+ *
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
+ */
+
 #ifndef _E_UTIL_H_
 #define _E_UTIL_H_
 
@@ -81,6 +104,8 @@ char     *e_read_file                                                      (cons
 int       e_write_file                                                     (const char        *filename,
 									    const char        *data,
 									    int                flags);
+int       e_write_file_mkstemp                                             (char              *filename,
+									    const char        *data);
 int       e_mkdir_hier                                                     (const char        *path,
 									    mode_t             mode);
 
@@ -227,6 +252,10 @@ void      e_marshal_INT__POINTER_POINTER_POINTER_POINTER_POINTER           (GtkO
 									    GtkSignalFunc      func,
 									    gpointer           func_data,
 									    GtkArg            *args);
+void      e_marshal_NONE__POINTER_POINTER_POINTER_BOOL	                   (GtkObject         *object,
+									    GtkSignalFunc      func,
+									    gpointer           func_data,
+									    GtkArg            *args);
 void      e_marshal_NONE__POINTER_INT_INT_INT                              (GtkObject         *object,
 									    GtkSignalFunc      func,
 									    gpointer           func_data,
